package pwc.taxtech.atms.common.ftp;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.net.ftp.FTP;
import org.apache.commons.net.ftp.FTPClient;
import org.apache.commons.net.ftp.FTPReply;
import org.apache.commons.pool2.PooledObject;
import org.apache.commons.pool2.PooledObjectFactory;
import org.apache.commons.pool2.impl.DefaultPooledObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import pwc.taxtech.atms.exception.ServiceException;

import java.io.IOException;

public class FtpClientFactory implements PooledObjectFactory<FTPClient> {
    private static final Logger logger = LoggerFactory.getLogger(FtpClientFactory.class);
    private FTPClientConfig config;

    public FtpClientFactory(FTPClientConfig config) {
        this.config = config;
    }

    public FTPClientConfig getConfig() {
        return config;
    }

    @Override
    public PooledObject<FTPClient> makeObject() throws Exception {
        FTPClient ftpClient = new FTPClient();
        if (null != config.getClientTimeout()) {
            ftpClient.setConnectTimeout(config.getClientTimeout());
        }
        ftpClient.connect(config.getHost(), config.getPort());
        int reply = ftpClient.getReplyCode();
        if (!FTPReply.isPositiveCompletion(reply)) {
            ftpClient.disconnect();
            logger.warn("FTPServer refused connection");
            throw new ServiceException("FTPServer refused connection");
        }
        boolean result = ftpClient.login(StringUtils.defaultString(config.getUsername()),
                StringUtils.defaultString(config.getPassword()));
        if (!result) {
            throw new ServiceException("ftpClient登陆失败! userName:" + config.getUsername() + " ; password:" + config.getPassword());
        }
        ftpClient.setFileType(FTP.BINARY_FILE_TYPE);
        if (null != config.getTransferFileType()) {
            ftpClient.setFileType(config.getTransferFileType());
        }
        ftpClient.setBufferSize(1024);
        ftpClient.setControlEncoding(StringUtils.defaultString(config.getEncoding(), "UTF-8"));
        ftpClient.enterLocalPassiveMode();
//            if (StringUtils.equals(config.getPassiveMode(), "true")) {
//                ftpClient.enterLocalPassiveMode();
//            }

        logger.debug("mk objec ftp client {}", ftpClient.toString());
        return new DefaultPooledObject<>(ftpClient);

    }

    @Override
    public void destroyObject(PooledObject<FTPClient> pooledObject) throws Exception {
        FTPClient ftpClient = pooledObject.getObject();
        try {
            if (ftpClient != null && ftpClient.isConnected()) {
                logger.debug("destroy ftp client {}", ftpClient.toString());
                ftpClient.logout();
            }
        } finally {
            try {
                if (ftpClient != null && ftpClient.isConnected())
                    ftpClient.disconnect();
            } catch (IOException io) {
                io.printStackTrace();
            }
        }
    }

    @Override
    public boolean validateObject(PooledObject<FTPClient> pooledObject) {
        try {
            logger.debug("validateObject {}", pooledObject.getObject().toString());
            return pooledObject.getObject().sendNoOp() && pooledObject.getObject().isAvailable()
                    && pooledObject.getObject().isConnected();
        } catch (IOException e) {
            throw new RuntimeException("Failed to validate client: " + e, e);
        }
    }

    @Override
    public void activateObject(PooledObject<FTPClient> pooledObject) throws Exception {
        logger.debug("activateObject {}", pooledObject.getObject().toString());
    }

    @Override
    public void passivateObject(PooledObject<FTPClient> pooledObject) throws Exception {
        logger.debug("passivateObject {}", pooledObject.getObject().toString());
    }

}
