package pwc.taxtech.atms.vat.service.impl;

import org.apache.commons.lang3.StringUtils;
import org.apache.poi.openxml4j.exceptions.InvalidFormatException;
import org.apache.poi.ss.formula.functions.FreeRefFunction;
import org.apache.poi.ss.formula.udf.AggregatingUDFFinder;
import org.apache.poi.ss.formula.udf.DefaultUDFFinder;
import org.apache.poi.ss.formula.udf.UDFFinder;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.FormulaEvaluator;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.ss.usermodel.WorkbookFactory;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import pwc.taxtech.atms.common.POIUtil;
import pwc.taxtech.atms.constant.enums.CellDataSourceType;
import pwc.taxtech.atms.dao.ProjectMapper;
import pwc.taxtech.atms.dto.vatdto.CellCalcInfoDto;
import pwc.taxtech.atms.dto.vatdto.CellTemplateConfigGroupDto;
import pwc.taxtech.atms.dto.vatdto.PeriodCellTemplateConfigExtendDto;
import pwc.taxtech.atms.entitiy.Project;
import pwc.taxtech.atms.vat.entity.*;
import pwc.taxtech.atms.vat.service.ReportGenerator;
import pwc.taxtech.atms.vat.service.impl.report.functions.BB;
import pwc.taxtech.atms.vat.service.impl.report.functions.FSJZ;
import pwc.taxtech.atms.vat.service.impl.report.functions.FormulaContext;
import pwc.taxtech.atms.vat.service.impl.report.functions.ND;
import pwc.taxtech.atms.vat.service.impl.report.functions.SGSR;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.stream.Collectors;

import static pwc.taxtech.atms.constant.Constant.EMPTY;

@Service
public class ReportGeneratorImpl extends VatAbstractService implements ReportGenerator {

    private static final Logger logger = LoggerFactory.getLogger(ReportGeneratorImpl.class);
    @Autowired
    private ProjectMapper projectMapper;

    @Override
    public String generateData(String projectId, List<Long> templateIds, Boolean ifDeleteManualDataSource,
                               List<String> exceptCellTemplates, Integer periodParam, String generator) {
        Date createTime = new Date();
        int period = periodParam != null ? periodParam : 0;

        Project project = projectMapper.selectByPrimaryKey(projectId);

        PeriodTemplateExample periodTemplateExample = new PeriodTemplateExample();
        periodTemplateExample.createCriteria().andIdIn(templateIds);
        List<PeriodTemplate> periodTemplateList = periodTemplateMapper.selectByExample(periodTemplateExample);
        List<Long> periodTemplateIDList = periodTemplateList.stream()
                .map(PeriodTemplate::getTemplateId)
                .collect(Collectors.toList());

        Long templateGroupID = periodTemplateList.size() > 0 ? periodTemplateList.get(0).getTemplateGroupId() : 0;

        PeriodCellTemplateExample periodCellTemplateExample = new PeriodCellTemplateExample();
        periodCellTemplateExample.createCriteria().andReportTemplateIdIn(periodTemplateIDList);
        List<PeriodCellTemplate> periodCellTemplateList = periodCellTemplateMapper.selectByExample(periodCellTemplateExample);

        PeriodCellTemplateConfigExample periodCellTemplateConfigExample = new PeriodCellTemplateConfigExample();
        // only get formula config
        periodCellTemplateConfigExample.createCriteria().andReportTemplateIdIn(periodTemplateIDList);
        List<PeriodCellTemplateConfig> periodCellTemplateConfigList =
                periodCellTemplateConfigMapper.selectByExample(periodCellTemplateConfigExample);

        Map<String, String> templateCodeAndPath = new HashMap<>();
        periodTemplateList.forEach(a -> templateCodeAndPath.put(a.getCode(), getTemplatePath(a.getPath())));
        Optional<Workbook> workbook = createWorkBookWithExcelFileList(templateCodeAndPath);

        if (workbook != null) {
            Workbook newWorkbook = workbook.get();
            FormulaContext formulaContext = new FormulaContext();
            formulaContext.setFormulaAgent(formulaAgent);
            formulaContext.setPeriod(period);
            formulaContext.setReportTemplateGroupID(templateGroupID);
            formulaContext.setProjectID(project.getID());
            formulaContext.setYear(project.getYear());
            formulaContext.setOrganizationID(project.getOrganizationID());

            addFunctionsToWorkbook(newWorkbook, formulaContext);
            int sheetCount = newWorkbook.getNumberOfSheets();
            for (int i = 0; i < sheetCount; i++) {
                Sheet sheet = newWorkbook.getSheetAt(i);
                String code = sheet.getSheetName();
                Optional<PeriodTemplate> periodTemplate = periodTemplateList.stream()
                        .filter(a -> a.getCode().equals(code))
                        .findFirst();
                Long templateID;
                if (periodTemplate.isPresent()) {
                    templateID = periodTemplate.get().getTemplateId();
                } else {
                    templateID = 0L;
                }

                if (templateID > 0) {
                    //get cell template and cell template config with templateID
                    Map<PeriodCellTemplate, PeriodCellTemplateConfig> cellTemplatePeriodCellTemplateConfigMap = new HashMap<>();
                    List<PeriodCellTemplate> tempPeriodCellTemplateList = periodCellTemplateList.stream()
                            .filter(a -> a.getReportTemplateId().equals(templateID))
                            .collect(Collectors.toList());
                    tempPeriodCellTemplateList.forEach(a -> {
                        Optional<PeriodCellTemplateConfig> tempCellTemplateConfig = periodCellTemplateConfigList.stream()
                                .filter(item -> item.getCellTemplateId().equals(a.getCellTemplateId())
                                        && item.getDataSourceType().equals(CellDataSourceType.Formula.getCode()))
                                .findFirst();
                        tempCellTemplateConfig.ifPresent(periodCellTemplateConfig ->
                                cellTemplatePeriodCellTemplateConfigMap.put(a, periodCellTemplateConfig));
                    });

                    cellTemplatePeriodCellTemplateConfigMap.forEach((k, v) -> {
                        Row row = sheet.getRow(k.getRowIndex());
                        if (null == row) {
                            sheet.createRow(k.getRowIndex());
                        }
                        row = sheet.getRow(k.getRowIndex());

                        Cell cell = row.getCell(k.getColumnIndex());
                        if (null == cell) {
                            row.createCell(k.getColumnIndex());
                        }
                        cell = row.getCell(k.getColumnIndex());

                        //todo:后面单独处理kv的公式
                        if (StringUtils.isNotBlank(v.getFormula()) && !v.getFormula().contains("@")) {
                            cell.setCellFormula(v.getFormula());
                        }
                    });
                }
            }
            FormulaEvaluator evaluator = newWorkbook.getCreationHelper().createFormulaEvaluator();
            evaluator.evaluateAll();

            //todo:loop all the cell which existing in the cellTemplateConfig, save the data to DB
            for (int i = 0; i < sheetCount; i++) {
                Sheet sheet = newWorkbook.getSheetAt(i);
                String code = sheet.getSheetName();
                Optional<PeriodTemplate> periodTemplate = periodTemplateList.stream()
                        .filter(a -> a.getCode().equals(code))
                        .findFirst();
                Long templateID;
                if (periodTemplate.isPresent()) {
                    templateID = periodTemplate.get().getTemplateId();
                } else {
                    templateID = 0L;
                }

                if (templateID > 0) {
                    //todo: create report into DB
                    Report report = new Report();
                    Long reportID = distributedIDService.nextId();
                    report.setId(reportID);
                    report.setTemplateId(templateID);
                    report.setPeriod(period);
                    report.setProjectId(projectId);
                    report.setCreateBy("Admin");
                    report.setCreateTime(createTime);
                    report.setUpdateBy("Admin");
                    report.setUpdateTime(createTime);
                    reportMapper.insertSelective(report);

                    List<PeriodCellTemplateConfig> periodCellTemplateConfigs = periodCellTemplateConfigList.stream()
                            .filter(a -> a.getReportTemplateId().equals(templateID))
                            .collect(Collectors.toList());

                    //update formulablock table reportid field
                    List<Long> cellTemplateConfigIds = periodCellTemplateConfigs.stream()
                            .map(PeriodCellTemplateConfig::getCellTemplateId)
                            .collect(Collectors.toList());
                    periodFormulaBlockMapper.updateReportId(reportID, cellTemplateConfigIds, period);

                    for (PeriodCellTemplateConfig periodCellTemplateConfig : periodCellTemplateConfigs) {
                        Optional<PeriodCellTemplate> tempPeriodCellTemplate = periodCellTemplateList.stream()
                                .filter(a -> a.getCellTemplateId().equals(periodCellTemplateConfig.getCellTemplateId()))
                                .findFirst();
                        if (tempPeriodCellTemplate.isPresent()) {
                            CellData cellData = new CellData();
                            cellData.setId(distributedIDService.nextId());
                            cellData.setReportId(reportID);
                            cellData.setCellTemplateId(tempPeriodCellTemplate.get().getCellTemplateId());
                            String data;
                            if (sheet.getRow(tempPeriodCellTemplate.get().getRowIndex()) != null
                                    && sheet.getRow(tempPeriodCellTemplate.get().getRowIndex())
                                    .getCell(tempPeriodCellTemplate.get().getColumnIndex()) != null) {
                                data = Double.toString(sheet.getRow(tempPeriodCellTemplate.get().getRowIndex())
                                        .getCell(tempPeriodCellTemplate.get().getColumnIndex()).getNumericCellValue());
                            } else {
                                data = EMPTY;
                            }
                            cellData.setData(data);
                            PeriodFormulaBlockExample periodFormulaBlockExample = new PeriodFormulaBlockExample();
                            periodFormulaBlockExample.createCriteria().andPeriodEqualTo(period)
                                    .andCellTemplateIdEqualTo(tempPeriodCellTemplate.get().getCellTemplateId());


                            //todo: 这里可能一个单元格不止一个公式，需要根据公式的内容来进行替换具体的解析公式
                            Optional<PeriodFormulaBlock> periodFormulaBlock =
                                    periodFormulaBlockMapper.selectByExample(periodFormulaBlockExample).stream().findFirst();

                            if (periodFormulaBlock.isPresent()) {
                                cellData.setFormulaExp(periodFormulaBlock.get().getFormulaExpression());
                            } else {
                                cellData.setFormulaExp(EMPTY);
                            }
                            cellData.setCreateBy("Admin");
                            cellData.setCreateTime(createTime);
                            cellData.setUpdateBy("Admin");
                            cellData.setUpdateTime(createTime);
                            cellDataMapper.insertSelective(cellData);
                        }
                    }
                }
            }

        } else {
            return "GenerateReport failed";
        }
        return "generate report successful";
    }

    private String getTemplatePath(String path) {
        String result;
        String PREFIX_VALUE = "~";
        if (path.startsWith(PREFIX_VALUE)) {
            result = path.substring(1, path.length());
        } else {
            result = path;
        }
        return result;
    }

    /**
     * 根据传入的模板code和模板路径创建workbook
     *
     * @param templates 模板code和模板路径 键值对
     * @return 工作簿workbook
     */
    private Optional<Workbook> createWorkBookWithExcelFileList(Map<String, String> templates) {
        Workbook workbook = new XSSFWorkbook();
        try {
            String filePath = this.getClass().getResource("").toURI().getPath();
            String tempPath = filePath.substring(0, filePath.indexOf("classes") + "\\classes".length());
            templates.forEach((code, path) -> {
                File file = new File(tempPath + path);
                try {
                    Workbook tWorkbook = WorkbookFactory.create(file);
                    POIUtil.cloneSheet(tWorkbook.getSheetAt(0), workbook.createSheet(code));
                } catch (FileNotFoundException e) {
                    e.printStackTrace();
                } catch (InvalidFormatException e) {
                    e.printStackTrace();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            });
            return Optional.of(workbook);
        } catch (Exception e) {
            logger.error("createWorkBookWithExcelFileList error.", e);
        }
        return Optional.empty();
    }

    /**
     * 注册所有的自定义方法到工作簿
     *
     * @param workbook 工作簿
     */
    private void addFunctionsToWorkbook(Workbook workbook, FormulaContext formulaContext) {
        String[] functionNames = {"SGSR", "FSJZ", "ND", "BB"};
        FreeRefFunction[] functionImpls = {new SGSR(formulaContext), new FSJZ(formulaContext), new ND(formulaContext),
                new BB(formulaContext)};
        UDFFinder udfs = new DefaultUDFFinder(functionNames, functionImpls);
        UDFFinder udfToolpack = new AggregatingUDFFinder(udfs);
        workbook.addToolPack(udfToolpack);
    }

    public List<CellCalcInfoDto> getCellCalcInfo(List<Long> templateIDList, Integer periodParam) {
        //already finished by 20180711
        int period = 0;
        if (periodParam != null) {
            period = periodParam;
        }

        List<PeriodCellTemplateConfigExtendDto> periodCellTemplateConfigExtendDtos =
                periodCellTemplateConfigMapper.getPeriodCellTemplateConfigExtendDtos(templateIDList, period);

        List<CellCalcInfoDto> cellCalcInfoDtos = new ArrayList<>();
        periodCellTemplateConfigExtendDtos.stream().collect(Collectors.groupingBy(a ->
                new CellTemplateConfigGroupDto(a.getColumnIndex(), a.getRowIndex()
                        , a.getColumnName(), a.getRowName(), a.getDataType(), a.getIsReadOnly()
                        , a.getCellTemplateID(), a.getReportTemplateID(), a.getComment()))).forEach((x, y) -> {
            CellCalcInfoDto cellCalcInfoDto = new CellCalcInfoDto();
            cellCalcInfoDto.setColumnIndex(x.getColumnIndex());
            cellCalcInfoDto.setColumnName(x.getColumnName());
            cellCalcInfoDto.setRowIndex(x.getRowIndex());
            cellCalcInfoDto.setRowName(x.getRowName());
            cellCalcInfoDto.setFormula(convertListToString(y.stream()
                    .map(PeriodCellTemplateConfig::getFormula).collect(Collectors.toList())));
            cellCalcInfoDto.setParsedFormula(convertListToString(y.stream()
                    .map(PeriodCellTemplateConfig::getParsedFormula).collect(Collectors.toList())));
            //todo: add validation at here later
            cellCalcInfoDto.setCellTemplateID(x.getCellTemplateID());
            cellCalcInfoDto.setReportTemplateID(x.getReportTemplateID());
            cellCalcInfoDto.setDataType(x.getDataType());
            cellCalcInfoDto.setIsReadOnly(x.getIsReadOnly());
            cellCalcInfoDto.setComment(x.getComment());
            cellCalcInfoDto.setConfigList(y.stream()
                    .map(PeriodCellTemplateConfigExtendDto::getPeriodCellTemplateConfig)
                    .collect(Collectors.toList()));
            cellCalcInfoDtos.add(cellCalcInfoDto);
        });
        return cellCalcInfoDtos;
    }

    private String convertListToString(List<String> list) {
        StringBuilder stringBuilder = new StringBuilder();
        list.forEach(s -> {
            if (StringUtils.isNotBlank(s)) {
                stringBuilder.append(s);
                stringBuilder.append("+");
            }
        });
        if (stringBuilder.length() > 0) {
            stringBuilder.deleteCharAt(stringBuilder.length() - 1);
        }
        return stringBuilder.toString();
    }
}
