package pwc.taxtech.atms.service.impl;

import static org.assertj.core.api.Assertions.assertThat;
import static org.junit.Assert.assertEquals;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;

import pwc.taxtech.atms.CommonIT;
import pwc.taxtech.atms.common.CommonConstants;
import pwc.taxtech.atms.common.OperationModule;
import pwc.taxtech.atms.common.message.AreaMessage;
import pwc.taxtech.atms.common.message.RegionMessage;
import pwc.taxtech.atms.dto.AreaDto;
import pwc.taxtech.atms.dto.OperationResultDto;
import pwc.taxtech.atms.entitiy.Area;
import pwc.taxtech.atms.entitiy.OperationLogBasicData;
import pwc.taxtech.atms.entitiy.OperationLogBasicDataExample;
import pwc.taxtech.atms.security.JwtAuthenticationProvider;
import pwc.taxtech.atms.security.JwtAuthenticationToken;
import pwc.taxtech.atms.service.AreaService;

public class AreaServiceIT extends CommonIT {

    @Autowired
    private AreaService areaService;

    @Autowired
    private JwtAuthenticationProvider jwtAuthenticationProvider;

    @Before
    public void setUp() {

        // delete all from Area
        cleanUpAllTables();
    }

    @Test
    public void findTopAreasTest() {

        // Data preparation
        for (int i = 0; i < 4; i++) {
            Area area = new Area();
            area.setID("area" + (i + 1));
            area.setName("area" + (i + 1));
            area.setIsActive(true);
            if (i % 2 == 1) {
                area.setParentID("area" + i);
            }
            areaMapper.insert(area);
        }

        List<Area> areaList = areaService.findTopAreas();
        assertEquals(2, areaList.size());
    }

    @Test
    public void findAreasByParentIdTest() {
        // Data preparation
        for (int i = 0; i < 4; i++) {
            Area area = new Area();
            area.setID("area" + (i + 1));
            area.setName("area" + (i + 1));
            area.setIsActive(true);
            if (i >= 1) {
                area.setParentID("area1");
            }
            areaMapper.insert(area);
        }
        List<Area> areaList = areaService.getSubAreaListIngoreActive("area1");
        assertEquals(3, areaList.size());
    }

    @Test
    public void setIsActiveTestDeactivateSingle() {

        // data preparation
        execSqlFile("sql/area_data_update_test.sql");
        // get current user name
        Authentication request = new JwtAuthenticationToken("xxxx");
        Authentication authenticaiton = jwtAuthenticationProvider.authenticate(request);
        SecurityContextHolder.getContext().setAuthentication(authenticaiton);

        AreaDto areaDto = new AreaDto();
        areaDto.setID("359667ee-bf7b-47bc-95b4-1c6e00f1de8d");
        areaDto.setIsActive(CommonConstants.DEACTIVE_STATUS);
        OperationResultDto<List<String>> operationResult = areaService.setIsActive(areaDto);

        // check area
        Area expectedArea = areaMapper.selectByPrimaryKey("359667ee-bf7b-47bc-95b4-1c6e00f1de8d");
        Assert.assertEquals(CommonConstants.DEACTIVE_STATUS, expectedArea.getIsActive());
        // check response
        Assert.assertEquals(true, operationResult.getResult());
        // check operation log
        OperationLogBasicDataExample example = new OperationLogBasicDataExample();
        List<OperationLogBasicData> operationLogs = operationLogBasicDataMapper.selectByExample(example);
        Assert.assertEquals(1, operationLogs.size());
        OperationLogBasicData log = operationLogs.get(0);
        Assert.assertEquals(log.getOperationObject(), expectedArea.getName());
        Assert.assertEquals(CommonConstants.ACTIVE_STATUS.toString(), log.getOriginalState());
        Assert.assertEquals(CommonConstants.DEACTIVE_STATUS.toString(), log.getUpdateState());
        Assert.assertEquals(OperationModule.BasicDataArea.name(), log.getModuleName());
    }

    @Test
    public void setIsActiveTestCascadeDeactivate() {

        // data preparation
        execSqlFile("sql/area_data_update_test.sql");
        // get current user name
        Authentication request = new JwtAuthenticationToken("xxxx");
        Authentication authenticaiton = jwtAuthenticationProvider.authenticate(request);
        SecurityContextHolder.getContext().setAuthentication(authenticaiton);

        AreaDto areaDto = new AreaDto();
        areaDto.setID("141eee99-4046-473e-a8b7-7ccd25892b71");
        areaDto.setIsActive(CommonConstants.DEACTIVE_STATUS);
        OperationResultDto<List<String>> operationResult = areaService.setIsActive(areaDto);

        // check area
        Area expectedArea = areaMapper.selectByPrimaryKey("141eee99-4046-473e-a8b7-7ccd25892b71");
        Assert.assertEquals(CommonConstants.DEACTIVE_STATUS, expectedArea.getIsActive());
        List<Area> allSubAreas = areaService.getAllSubAreas("141eee99-4046-473e-a8b7-7ccd25892b71");
        Assert.assertEquals(3, allSubAreas.size());
        Assert.assertArrayEquals(
                new Boolean[] { CommonConstants.DEACTIVE_STATUS, CommonConstants.DEACTIVE_STATUS,
                        CommonConstants.DEACTIVE_STATUS },
                allSubAreas.stream().map(area -> area.getIsActive()).collect(Collectors.toList()).toArray());

        // check response
        Assert.assertEquals(true, operationResult.getResult());

        // check operation log
        OperationLogBasicDataExample example = new OperationLogBasicDataExample();
        example.setOrderByClause("CreateTime ASC");
        List<OperationLogBasicData> operationLogs = operationLogBasicDataMapper.selectByExample(example);
        Assert.assertEquals(4, operationLogs.size());
        assertThat(operationLogs).extracting("originalState").contains(CommonConstants.ACTIVE_STATUS.toString());
        assertThat(operationLogs).extracting("updateState").contains(CommonConstants.DEACTIVE_STATUS.toString());
        assertThat(operationLogs).extracting("comment").contains(AreaMessage.AreaName + ":" + expectedArea.getName() + AreaMessage.DisableCascade);
        
        /*OperationLogBasicData targetArealog = operationLogs.get(0);
        Assert.assertEquals(targetArealog.getOperationObject(), expectedArea.getName());
        Assert.assertEquals(CommonConstants.ACTIVE_STATUS.toString(), targetArealog.getOriginalState());
        Assert.assertEquals(CommonConstants.DEACTIVE_STATUS.toString(), targetArealog.getUpdateState());
        Assert.assertEquals(OperationModule.BasicDataArea.name(), targetArealog.getModuleName());
        
        OperationLogBasicData cascadeArealog = operationLogs.get(1);
        Assert.assertEquals(allSubAreas.get(0).getName(), cascadeArealog.getOperationObject());
        Assert.assertEquals(CommonConstants.ACTIVE_STATUS.toString(), cascadeArealog.getOriginalState());
        Assert.assertEquals(CommonConstants.DEACTIVE_STATUS.toString(), cascadeArealog.getUpdateState());
        Assert.assertEquals(OperationModule.BasicDataArea.name(), cascadeArealog.getModuleName());
        Assert.assertEquals(AreaMessage.AreaName + ":" + expectedArea.getName() + AreaMessage.DisableCascade,
                cascadeArealog.getComment());*/
    }

    @Test
    public void setIsActiveTestCascadeDeactivateWithLinkedOrganization() {

        // data preparation
        execSqlFile("sql/area_data_update_test.sql");

        // get current user name
        Authentication request = new JwtAuthenticationToken("xxxx");
        Authentication authenticaiton = jwtAuthenticationProvider.authenticate(request);
        SecurityContextHolder.getContext().setAuthentication(authenticaiton);

        AreaDto areaDto = new AreaDto();
        areaDto.setID("141eb884-e9b9-4188-9a00-a3571ab1e7e9");
        areaDto.setIsActive(CommonConstants.DEACTIVE_STATUS);
        OperationResultDto<List<String>> operationResult = areaService.setIsActive(areaDto);

        // check response
        Assert.assertEquals(false, operationResult.getResult());
        Assert.assertEquals(RegionMessage.CannotStopRegionTips, operationResult.getResultMsg());
        Assert.assertEquals(Arrays.asList("华天证券有限公司"), operationResult.getData());

        // check operation log
        OperationLogBasicDataExample example = new OperationLogBasicDataExample();
        example.setOrderByClause("CreateTime ASC");
        List<OperationLogBasicData> operationLogs = operationLogBasicDataMapper.selectByExample(example);
        Assert.assertEquals(0, operationLogs.size());
    }

    @Test
    public void getAreaListTest() {
        // data preparation
        execSqlFile("sql/area_data.sql");

        List<AreaDto> areaDtoList = areaService.getAreaList(1);
        Assert.assertEquals(49, areaDtoList.size());

        areaDtoList = areaService.getAreaList(0);
        Assert.assertEquals(40, areaDtoList.size());
    }

    @Test
    public void getAreaTreeTest() {
        // data preparation
        execSqlFile("sql/area_data.sql");
        List<AreaDto> areaDtoTree = areaService.getAreaTree(1);
        Assert.assertEquals(31, areaDtoTree.size());

        areaDtoTree = areaService.getAreaTree(0);
        Assert.assertEquals(29, areaDtoTree.size());
    }

    @After
    public void tearDown() {

    }
}
