package pwc.taxtech.atms.service.impl;

import static org.junit.Assert.assertNotNull;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

import org.apache.commons.lang3.BooleanUtils;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;

import com.alibaba.fastjson.JSON;
import com.fasterxml.jackson.databind.ObjectMapper;

import pwc.taxtech.atms.CommonIT;
import pwc.taxtech.atms.common.CheckState;
import pwc.taxtech.atms.common.CommonUtils;
import pwc.taxtech.atms.common.RoleSourceEnum;
import pwc.taxtech.atms.common.UserStatus;
import pwc.taxtech.atms.dto.LoginInputDto;
import pwc.taxtech.atms.dto.LoginOutputDto;
import pwc.taxtech.atms.dto.OperationResultDto;
import pwc.taxtech.atms.dto.organization.DimensionRoleDto;
import pwc.taxtech.atms.dto.organization.OrgRoleDto;
import pwc.taxtech.atms.dto.organization.OrgRoleDtoList;
import pwc.taxtech.atms.dto.permission.PermissionDto;
import pwc.taxtech.atms.dto.permission.PermissionKeyDto;
import pwc.taxtech.atms.dto.permission.UserPermissionDto;
import pwc.taxtech.atms.dto.permission.UserPermissionKeyDto;
import pwc.taxtech.atms.dto.user.RoleInfo;
import pwc.taxtech.atms.dto.user.UpdateParam;
import pwc.taxtech.atms.dto.user.UserAndUserRoleSaveDto;
import pwc.taxtech.atms.dto.user.UserDto;
import pwc.taxtech.atms.dto.user.UserOrgRoleDto;
import pwc.taxtech.atms.dto.user.UserOrganizationDto;
import pwc.taxtech.atms.dto.user.UserRoleDimensionValueDto;
import pwc.taxtech.atms.dto.user.UserRoleDisplayInfo;
import pwc.taxtech.atms.entitiy.User;
import pwc.taxtech.atms.entitiy.UserDimensionValue;
import pwc.taxtech.atms.entitiy.UserDimensionValueOrg;
import pwc.taxtech.atms.entitiy.UserDimensionValueOrgExample;
import pwc.taxtech.atms.entitiy.UserDimensionValueRole;
import pwc.taxtech.atms.entitiy.UserDimensionValueRoleExample;
import pwc.taxtech.atms.entitiy.UserOrganization;
import pwc.taxtech.atms.security.JwtAuthenticationProvider;
import pwc.taxtech.atms.security.JwtAuthenticationToken;
import pwc.taxtech.atms.service.UserRoleService;
import pwc.taxtech.atms.service.UserService;

public class UserServiceIT extends CommonIT {
    private static final Logger logger = LoggerFactory.getLogger(UserServiceIT.class);
    @Autowired
    private UserService userService;
    @Autowired
    private UserRoleService userRoleService;
    @Autowired
    private ObjectMapper objectMapper;
    @Autowired
    private JwtAuthenticationProvider jwtAuthenticationProvider;

    @Before
    public void setUp() {
        cleanUpAllTables();
        // get current user name
        Authentication request = new JwtAuthenticationToken("xxxx");
        Authentication authenticaiton = jwtAuthenticationProvider.authenticate(request);
        SecurityContextHolder.getContext().setAuthentication(authenticaiton);
    }

    @Test(expected = IllegalArgumentException.class)
    public void loginWithNull() {
        LoginInputDto input = new LoginInputDto();
        OperationResultDto<LoginOutputDto> result = userService.login(input);
        logger.debug("print result json:{}", JSON.toJSONString(result, true));
    }

    @Test
    public void loginOk() {
        execSqlFile("sql/user_data.sql");
        LoginInputDto input = new LoginInputDto();
        input.setEmail("admin");
        input.setPassword("111111");
        OperationResultDto<LoginOutputDto> result = userService.login(input);
        Assert.assertTrue(result.getResult());
        Assert.assertEquals(CheckState.Success.value(), result.getData().getCheckState());
    }

    @Test
    public void loginBadPassword() {
        execSqlFile("sql/user_data.sql");
        LoginInputDto input = new LoginInputDto();
        input.setEmail("admin");
        input.setPassword("badPassword");
        OperationResultDto<LoginOutputDto> result = userService.login(input);
        Assert.assertFalse(result.getResult());
        Assert.assertEquals(CheckState.WrongPassword.value(), result.getData().getCheckState());
        // logger.debug("print result json:{}", JSON.toJSONString(result, true));
        // {
        // "data":{
        // "CheckState":4
        // },
        // "result":false
        // }
    }

    @Test
    public void loginUserNameNotExist() {
        execSqlFile("sql/user_data.sql");
        LoginInputDto input = new LoginInputDto();
        input.setEmail("adminxxx");
        input.setPassword("na");
        OperationResultDto<LoginOutputDto> result = userService.login(input);
        Assert.assertFalse(result.getResult());
        Assert.assertEquals(CheckState.UserNameNotExist.value(), result.getData().getCheckState());
    }

    @Test
    public void tryWrongPwd6Times() {
        execSqlFile("sql/user_data.sql");
        User tempUser0 = userMapper.selectByUserNameIgnoreCase("admin");
        Assert.assertEquals(UserStatus.Active.value(), tempUser0.getStatus());

        for (int i = 0; i < 5; i++) {
            LoginInputDto input = new LoginInputDto();
            input.setEmail("admin");
            input.setPassword("badPassword");
            OperationResultDto<LoginOutputDto> result = userService.login(input);
            Assert.assertFalse(result.getResult());
            // logger.debug("" + result.getData().getCheckState());
            Assert.assertEquals(CheckState.WrongPassword.value(), result.getData().getCheckState());
        }
        {
            LoginInputDto input = new LoginInputDto();
            input.setEmail("admin");
            input.setPassword("badPassword");
            OperationResultDto<LoginOutputDto> result = userService.login(input);
            Assert.assertFalse(result.getResult());
            // logger.debug("" + result.getData().getCheckState());
            Assert.assertEquals(CheckState.Inactive.value(), result.getData().getCheckState());
        }
        User tempUser = userMapper.selectByUserNameIgnoreCase("admin");
        logger.debug("print user.status:" + tempUser.getStatus());
        Assert.assertEquals(UserStatus.Locked.value(), tempUser.getStatus());

    }

    @Test
    public void getUserPermissionTest() {
        execSqlFile("sql/user_data.sql");
        execSqlFile("sql/userPermission_data.sql");
        UserPermissionDto userPermissionDto = userService.getUserPermission("admin");
        Assert.assertNotNull(userPermissionDto);
        Assert.assertNotNull(userPermissionDto.getPermissionList());
        Assert.assertNotNull(userPermissionDto.getNavigationUrlList());

        Assert.assertTrue(userPermissionDto.getIsAdmin());
        Assert.assertTrue(userPermissionDto.getIsSuperAdmin());
        Assert.assertEquals("admin", userPermissionDto.getUserName());
        Assert.assertEquals("66933E7B-DA75-4B2E-B7D6-AB65DCA20D50", userPermissionDto.getID());

        Assert.assertEquals(71, userPermissionDto.getPermissionList().size());
        Assert.assertEquals(8, userPermissionDto.getNavigationUrlList().size());
        List<PermissionDto> permissionDtos = userPermissionDto.getPermissionList().stream()
                .filter(p -> "a856fc05-e581-45af-bc51-951ffd836814".equals(p.getID())).collect(Collectors.toList());
        Assert.assertEquals(1, permissionDtos.size());
        Assert.assertEquals("02.004.002", permissionDtos.get(0).getCode());
    }

    @Test
    public void getUserPermissionKeyTest() {
        execSqlFile("sql/user_data.sql");
        execSqlFile("sql/userPermission_data.sql");
        UserPermissionKeyDto userPermissionKeyDto = userService.getUserPermissionKey("admin");
        Assert.assertNotNull(userPermissionKeyDto);
        Assert.assertNotNull(userPermissionKeyDto.getPermissionList());
        Assert.assertNotNull(userPermissionKeyDto.getNavigationUrlList());

        Assert.assertTrue(userPermissionKeyDto.getIsAdmin());
        Assert.assertTrue(userPermissionKeyDto.getIsSuperAdmin());
        Assert.assertEquals("admin", userPermissionKeyDto.getUserName());
        Assert.assertEquals("66933E7B-DA75-4B2E-B7D6-AB65DCA20D50", userPermissionKeyDto.getID());

        Assert.assertEquals(71, userPermissionKeyDto.getPermissionList().size());
        Assert.assertEquals(7, userPermissionKeyDto.getNavigationUrlList().size());
        List<PermissionKeyDto> permissionDtos = userPermissionKeyDto.getPermissionList().stream()
                .filter(p -> "a856fc05-e581-45af-bc51-951ffd836814".equals(p.getID())).collect(Collectors.toList());
        Assert.assertEquals(1, permissionDtos.size());
        Assert.assertEquals("02.004.002", permissionDtos.get(0).getCode());
    }

    @Test
    public void getAllUserRoleListTest() {
        execSqlFile("sql/user_role_list_data.sql");
        List<UserRoleDisplayInfo> userRoleDisplayInfos = userRoleService.getAllUserRoleList("2");

        Assert.assertNotNull(userRoleDisplayInfos);
        Assert.assertEquals(1, userRoleDisplayInfos.size());
        UserRoleDisplayInfo userRoleDisplayInfo = userRoleDisplayInfos.get(0);
        Assert.assertEquals("90f39fbc-e91f-43c5-b3e5-e695dd423e41", userRoleDisplayInfo.getUserID());
        Assert.assertEquals("bella验证世茂", userRoleDisplayInfo.getUserName());
        Assert.assertEquals("1221321@com", userRoleDisplayInfo.getEmail());
        Integer status = 1;
        Assert.assertEquals(status, userRoleDisplayInfo.getStatus());
        Assert.assertEquals("22336432-cd75-4359-9a70-41f5e7755a25", userRoleDisplayInfo.getOrganizationID());
        Assert.assertNull(userRoleDisplayInfo.getServiceTypeID());
        Assert.assertEquals("90f39fbc-e91f-43c5-b3e5-e695dd423e41", userRoleDisplayInfo.getID());

        Assert.assertNotNull(userRoleDisplayInfo.getRoleInfoList());
        Assert.assertEquals(1, userRoleDisplayInfo.getRoleInfoList().size());
        RoleInfo roleInfo = userRoleDisplayInfo.getRoleInfoList().get(0);
        Assert.assertEquals("Admin系统管理员", roleInfo.getName());
        Assert.assertEquals(RoleSourceEnum.OriginalLevel.value(), roleInfo.getRoleSource());
        Assert.assertEquals("1CC31CE3-DE83-4FCC-8D96-F09CFBAE8783", roleInfo.getID());
    }

    @Test
    public void getUserTest() {
        execSqlFile("sql/user_role_list_data.sql");
        UserDto userDto = userService.getUserByID("90f39fbc-e91f-43c5-b3e5-e695dd423e41");

        Assert.assertNotNull(userDto);
        Assert.assertEquals("bella验证世茂", userDto.getUserName());
        Assert.assertEquals("1221321@com", userDto.getEmail());
        Integer value = 3;
        Assert.assertEquals(value, userDto.getLoginType());
        value = 1;
        Assert.assertEquals(value, userDto.getStatus());
        Assert.assertNotNull(userDto.getCreateTime());
        Assert.assertNotNull(userDto.getUpdateTime());
        Assert.assertTrue(userDto.getIsAdmin());
        value = 0;
        Assert.assertEquals(value, userDto.getAttemptTimes());
        Assert.assertEquals("22336432-cd75-4359-9a70-41f5e7755a25", userDto.getOrganizationID());
        Assert.assertEquals("90f39fbc-e91f-43c5-b3e5-e695dd423e41", userDto.getID());
    }

    @Test
    public void getSingleUserByUserIdTest() {
        execSqlFile("sql/user_role_list_data.sql");
        UserAndUserRoleSaveDto userAndUserRoleSaveDto = userRoleService
                .getSingleUserByUserId("90f39fbc-e91f-43c5-b3e5-e695dd423e41");

        Assert.assertNotNull(userAndUserRoleSaveDto);
        Assert.assertEquals("bella验证世茂", userAndUserRoleSaveDto.getUserName());
        Assert.assertEquals("1221321@com", userAndUserRoleSaveDto.getEmail());
        Integer value = 3;
        Assert.assertEquals(value, userAndUserRoleSaveDto.getLoginType());
        value = 1;
        Assert.assertEquals(value, userAndUserRoleSaveDto.getStatus());
        Assert.assertNotNull(userAndUserRoleSaveDto.getCreateTime());
        Assert.assertNotNull(userAndUserRoleSaveDto.getUpdateTime());
        Assert.assertTrue(userAndUserRoleSaveDto.getIsAdmin());
        value = 0;
        Assert.assertEquals(value, userAndUserRoleSaveDto.getAttemptTimes());
        Assert.assertEquals("22336432-cd75-4359-9a70-41f5e7755a25", userAndUserRoleSaveDto.getOrganizationID());
        Assert.assertEquals("90f39fbc-e91f-43c5-b3e5-e695dd423e41", userAndUserRoleSaveDto.getID());

        Assert.assertNotNull(userAndUserRoleSaveDto.getRoleIDs());
        Assert.assertEquals(1, userAndUserRoleSaveDto.getRoleIDs().size());
        Assert.assertEquals("1CC31CE3-DE83-4FCC-8D96-F09CFBAE8783", userAndUserRoleSaveDto.getRoleIDs().get(0));

        Assert.assertNotNull(userAndUserRoleSaveDto.getRoleNames());
        Assert.assertEquals(1, userAndUserRoleSaveDto.getRoleNames().size());
        Assert.assertEquals("Admin系统管理员", userAndUserRoleSaveDto.getRoleNames().get(0));
    }

    @Test
    public void getUserRoleListByUserIDTest() {
        execSqlFile("sql/user_role_list_data.sql");
        UserRoleDisplayInfo userRoleDisplayInfo = userRoleService
                .getUserRoleListByUserID("90f39fbc-e91f-43c5-b3e5-e695dd423e41");
        Assert.assertNotNull(userRoleDisplayInfo);
        Assert.assertEquals("90f39fbc-e91f-43c5-b3e5-e695dd423e41", userRoleDisplayInfo.getUserID());
        Assert.assertEquals("bella验证世茂", userRoleDisplayInfo.getUserName());
        Assert.assertEquals("1221321@com", userRoleDisplayInfo.getEmail());
        Integer status = 1;
        Assert.assertEquals(status, userRoleDisplayInfo.getStatus());
        Assert.assertEquals("22336432-cd75-4359-9a70-41f5e7755a25", userRoleDisplayInfo.getOrganizationID());
        Assert.assertNull(userRoleDisplayInfo.getServiceTypeID());
        Assert.assertEquals("90f39fbc-e91f-43c5-b3e5-e695dd423e41", userRoleDisplayInfo.getID());

        Assert.assertNotNull(userRoleDisplayInfo.getRoleInfoList());
        Assert.assertEquals(1, userRoleDisplayInfo.getRoleInfoList().size());
        RoleInfo roleInfo = userRoleDisplayInfo.getRoleInfoList().get(0);
        Assert.assertEquals("Admin系统管理员", roleInfo.getName());
        Assert.assertEquals(RoleSourceEnum.OriginalLevel.value(), roleInfo.getRoleSource());
        Assert.assertEquals("1CC31CE3-DE83-4FCC-8D96-F09CFBAE8783", roleInfo.getID());
    }

    @Test
    public void getUserRoleByOrgIDUserIDTest() {
        execSqlFile("sql/user_data.sql");
        execSqlFile("sql/role_related_data.sql");
        UserOrganizationDto userOrganizationDto = userRoleService
                .getUserRoleByOrgID("66933E7B-DA75-4B2E-B7D6-AB65DCA20D50", "2d87ff73-2d13-43aa-851c-efe7b59e21af");
        Assert.assertNotNull(userOrganizationDto);
        Assert.assertNotNull(userOrganizationDto.getOrganizationName());
        Assert.assertNull(userOrganizationDto.getUserID());
        Assert.assertNull(userOrganizationDto.getOrganizationID());
        Assert.assertNull(userOrganizationDto.getHasOriginalRole());
        Assert.assertTrue(userOrganizationDto.getIsAccessible());
    }

    @Test
    public void getUserRoleByUserIDTest() {
        execSqlFile("sql/user_data.sql");
        execSqlFile("sql/role_related_data.sql");
        OrgRoleDtoList orgRoleDtoList = userRoleService.getUserRoleByUserID("66933E7B-DA75-4B2E-B7D6-AB65DCA20D50");
        Assert.assertEquals(4, orgRoleDtoList.getOrgDtoList().size());
        Assert.assertEquals(4, orgRoleDtoList.getOrgRoleInfoList().size());
        OrgRoleDto orgRoleDto = orgRoleDtoList.getOrgDtoList().stream().filter(s -> "741".equals(s.getOrgName()))
                .findFirst().orElse(null);
        Assert.assertNotNull(orgRoleDto);
        Assert.assertEquals("d0447c7b-5315-4da4-96f7-9ff8335f74fd", orgRoleDto.getOrgID());
        Assert.assertEquals(7, orgRoleDto.getRoleList().size());
    }

    @Test
    public void getOrgBuAreaIndustryUserTest() {
        execSqlFile("sql/user_org_area_industry_test.sql");
        List<DimensionRoleDto> dimensionRoleDtos = userRoleService
                .getOrgBuAreaIndustryUser("d0665e84-ab6e-4393-8b3c-666cff461efe");
        Assert.assertNotNull(dimensionRoleDtos);
        Assert.assertEquals(3, dimensionRoleDtos.size());
        DimensionRoleDto dimensionRoleDto = dimensionRoleDtos.stream()
                .filter(x -> "test8".equals(x.getDimensionValueName())).findFirst().orElse(null);
        Assert.assertNotNull(dimensionRoleDto);
        Assert.assertEquals("c61a5bd6-a996-4952-9869-d053995237e5", dimensionRoleDto.getDimensionID());
        Assert.assertEquals("事业部", dimensionRoleDto.getDimensionName());
        Assert.assertEquals("2399fb65-896a-4c99-9295-b73585bd1b50", dimensionRoleDto.getDimensionValueID());
        Assert.assertNotNull(dimensionRoleDto.getRoleList());
        Assert.assertEquals(8, dimensionRoleDto.getRoleList().size());
        Assert.assertTrue(dimensionRoleDto.getIsAccessible());
        Assert.assertTrue(dimensionRoleDto.getHasOriginalRole());
    }

    @SuppressWarnings("rawtypes")
    @Test
    public void enableOrDisableUserTest() {
        execSqlFile("sql/user_data.sql");
        execSqlFile("sql/role_related_data.sql");
        UpdateParam updateParam = new UpdateParam();
        updateParam.setUpdateType(0);
        updateParam.setUserID("66933E7B-DA75-4B2E-B7D6-AB65DCA20D50");
        OperationResultDto operationResultDto = userRoleService.enableOrDisableUser(updateParam);
        Assert.assertNotNull(operationResultDto);
        Assert.assertTrue(operationResultDto.getResult());
        Assert.assertNull(operationResultDto.getResultMsg());
        Assert.assertNull(operationResultDto.getReturnCode());
        Assert.assertNull(operationResultDto.getErrors());
        Assert.assertNull(operationResultDto.getData());
        User user = userMapper.selectByPrimaryKey("66933E7B-DA75-4B2E-B7D6-AB65DCA20D50");
        Assert.assertNotNull(user);
        Integer status = 0;
        Assert.assertEquals(status, user.getStatus());
    }

    @SuppressWarnings("rawtypes")
    @Test
    public void deleteUserRoleOrgTest() {
        execSqlFile("sql/user_data.sql");
        execSqlFile("sql/organization_filter_list_data.sql");
        UserOrganization userOrganization = new UserOrganization();
        String uuid = CommonUtils.getUUID();
        String userID = "66933E7B-DA75-4B2E-B7D6-AB65DCA20D50";
        String organizationID = "0a8fa9d8-db43-4820-94db-70ca02185b01";
        String dimensionID = CommonUtils.getUUID();
        String dimensionValueID = CommonUtils.getUUID();
        userOrganization.setID(uuid);
        userOrganization.setUserID(userID);
        userOrganization.setOrganizationID(organizationID);
        userOrganization.setIsAccessible(true);
        userOrganization.setHasOriginalRole(true);
        userOrganizationMapper.insert(userOrganization);

        UserDimensionValue userDimensionValue = new UserDimensionValue();
        userDimensionValue.setID(uuid);
        userDimensionValue.setUserID(userID);
        userDimensionValue.setDimensionID(dimensionID);
        userDimensionValue.setDimensionValueID(dimensionValueID);
        userDimensionValue.setIsAccessible(true);
        userDimensionValue.setHasOriginalRole(true);
        userDimensionValueMapper.insert(userDimensionValue);

        UserRoleDimensionValueDto userRoleDimensionValueDto = new UserRoleDimensionValueDto();
        userRoleDimensionValueDto.setDimensionValueID(dimensionValueID);
        userRoleDimensionValueDto.setDimensionID(dimensionID);
        userRoleDimensionValueDto.setUserID(userID);
        userRoleDimensionValueDto.setOrganizationID(organizationID);
        userRoleDimensionValueDto.setIsHeritable(false);
        List<UserRoleDimensionValueDto> list = new ArrayList<>();
        list.add(userRoleDimensionValueDto);
        OperationResultDto operationResultDto = userRoleService.deleteUserRoleOrg(list);
        Assert.assertTrue(BooleanUtils.isTrue(operationResultDto.getResult()));
        UserDimensionValueOrgExample userDimensionValueOrgExample = new UserDimensionValueOrgExample();
        userDimensionValueOrgExample.createCriteria().andOrganizationIDEqualTo(organizationID);
        List<UserDimensionValueOrg> userDimensionValueOrgList = userDimensionValueOrgMapper
                .selectByExample(userDimensionValueOrgExample);
        Assert.assertFalse(userDimensionValueOrgList.isEmpty());
        UserDimensionValueOrg userDimensionValueOrg = userDimensionValueOrgList.get(0);
        Assert.assertEquals(uuid, userDimensionValueOrg.getUserDimensionValueID());
        Assert.assertFalse(userDimensionValueOrg.getIsHeritable());
    }

    @Test
    public void getUserRoleByDimensionValueIDTest() {
        execSqlFile("sql/user_data.sql");
        execSqlFile("sql/role_related_data.sql");
        List<UserRoleDisplayInfo> userRoleDisplayInfos = userRoleService.getUserRoleByDimensionValueID(
                "c61a5bd6-a996-4952-9869-d053995237e6", "e2c4a67d-e7de-4037-939b-e6d437265ab0");
        Assert.assertNotNull(userRoleDisplayInfos);
        Assert.assertEquals(2, userRoleDisplayInfos.size());
        UserRoleDisplayInfo userRoleDisplayInfo = userRoleDisplayInfos.stream()
                .filter(s -> "Admin".equals(s.getUserName())).findFirst().orElse(null);
        Assert.assertNotNull(userRoleDisplayInfo);
        Assert.assertEquals("66933E7B-DA75-4B2E-B7D6-AB65DCA20D50", userRoleDisplayInfo.getUserID());
        Assert.assertNotNull(userRoleDisplayInfo.getRoleInfoList());
        Assert.assertEquals(7, userRoleDisplayInfo.getRoleInfoList().size());
        Assert.assertTrue(userRoleDisplayInfo.getIsAccessible());
        Assert.assertTrue(userRoleDisplayInfo.getHasOriginalRole());
    }

    @Test
    public void getSpecialUserRoleByDimensionValueIDTest() {
        // TODO
        execSqlFile("sql/user_data.sql");
        execSqlFile("sql/role_related_data.sql");
        List<UserRoleDisplayInfo> userRoleDisplayInfos = userRoleService.getSpecialUserRoleByDimensionValueID(
                "c61a5bd6-a996-4952-9869-d053995237e6", "e2c4a67d-e7de-4037-939b-e6d437265ab0");
        Assert.assertNotNull(userRoleDisplayInfos);
    }

    @Test
    public void getDimensionUserRoleListTest() {
        // TODO
        execSqlFile("sql/user_data.sql");
        execSqlFile("sql/role_related_data.sql");
        List<UserOrgRoleDto> userRoleDisplayInfos = userRoleService.getDimensionUserRoleList(
                "c61a5bd6-a996-4952-9869-d053995237e6", "e2c4a67d-e7de-4037-939b-e6d437265ab0");
        Assert.assertNotNull(userRoleDisplayInfos);
    }

    @SuppressWarnings("rawtypes")
    @Test
    public void updateUserRoleDimensionTest() {
        execSqlFile("sql/userRoleDimension_update_test.sql");
        List<UserRoleDimensionValueDto> userRoleList = new ArrayList<>();
        UserRoleDimensionValueDto userRoleDimensionValueDto1 = new UserRoleDimensionValueDto();
        userRoleDimensionValueDto1.setID("dd7808cd-e452-6ed2-4427-bb1a6c32cb0a");
        userRoleDimensionValueDto1.setDimensionValueID("2399fb65-896a-4c99-9295-b73585bd1b50");
        userRoleDimensionValueDto1.setDimensionID("c61a5bd6-a996-4952-9869-d053995237e5");
        userRoleDimensionValueDto1.setUserID("d0665e84-ab6e-4393-8b3c-666cff461efe");
        userRoleDimensionValueDto1.setRoleID("05D40C22-03F5-4A0F-A4B3-CEB1D7E6B9DC");
        userRoleDimensionValueDto1.setIsAdd(true);
        userRoleDimensionValueDto1.setIsAccessible(true);
        userRoleDimensionValueDto1.setHasOriginalRole(true);
        userRoleList.add(userRoleDimensionValueDto1);

        UserRoleDimensionValueDto userRoleDimensionValueDto2 = new UserRoleDimensionValueDto();
        userRoleDimensionValueDto2.setID("c5b0f6ed-97a1-314f-a8a0-9116f070294b");
        userRoleDimensionValueDto2.setDimensionValueID("2399fb65-896a-4c99-9295-b73585bd1b50");
        userRoleDimensionValueDto2.setDimensionID("c61a5bd6-a996-4952-9869-d053995237e5");
        userRoleDimensionValueDto2.setUserID("d0665e84-ab6e-4393-8b3c-666cff461efe");
        userRoleDimensionValueDto2.setRoleID("07CCEC5C-F059-4864-8009-D460689F1007");
        userRoleDimensionValueDto2.setIsAdd(true);
        userRoleDimensionValueDto2.setIsAccessible(true);
        userRoleDimensionValueDto2.setHasOriginalRole(true);
        userRoleList.add(userRoleDimensionValueDto2);

        OperationResultDto operationResultDto = userRoleService.updateUserRoleDimension(userRoleList);
        assertNotNull(operationResultDto);
        Assert.assertTrue(operationResultDto.getResult());
        UserDimensionValueRole us = userDimensionValueRoleMapper
                .selectByPrimaryKey("67DD050E-BC8D-4B5B-A3EA-86121533D07F");
        Assert.assertNull(us);

        List<UserDimensionValueRole> list = userDimensionValueRoleMapper
                .selectByExample(new UserDimensionValueRoleExample());
        Assert.assertEquals(2, list.size());
    }

    @Test
    public void updateUserTest() {
        // TODO
    }
    
    @Test
    public void updateUserRoleOrganizationTest() {
        // TODO
    }
    
    @Test
    public void addUserTest() {
        // TODO
    }
    
    @Test
    public void deleteUserDimensionValueTest() {
        // TODO
    }
    
    @Test
    public void deleteUserOrgTest() {
        // TODO
    }
    
    @Test
    public void updateUserRoleForOrgTest() {
        // TODO
    }
    
    @Test
    public void deleteUserRoleDimensionTest() {
        //TODO
    }
    
    @Test
    public void updateUserRoleForDimensionTest() {
        //TODO
    }
    

}
